use strict;
use ZRTeXtor ':all';
use ZRJCode ':all';
use constant { MAXGT => 78774, MAXFN => 13, MAXKFN => 2 };
use constant { GETA => '01-81AC' };
require 'map.pl';

sub main {
  j48_generate();
  zgu_generate();
}

##------ j48 encoding

sub j48_generate {
  my ($zvp, $vf, $tfm);
  $zvp = join('', u_zvp_prologue("GT-KANJI"), u_mapfont(),
                  j48_space(), u_type(), j48_character());
  write_whole_file("gtmc-r-j48.zvp", $zvp);
  return;
  ($vf, $tfm) = vf_form_ex(pl_parse($zvp)) or die textool_error();
  write_whole_file("gtmc-r-j48.vf", $vf, 1);
  write_whole_file("gtmc-r-j48.tfm", $tfm, 1);
}

sub j48_mapfont {
  my ($n, $fid, $fnm, @cnks);
  foreach $n (1 .. MAXFN) {
    $fid = $n - 1;
    $fnm = ($n > 1) ? sprintf("gtmc-r-g%02d", $n) : "r-gtmc-r-jy1";
    push(@cnks, <<"END");
(MAPFONT D $fid
   (FONTNAME $fnm)
   (FONTDSIZE R 10.0)
   (FONTAT R 1.0)
   )
END
  }
  return join('', @cnks);
}

sub j48_space {
  my $gc = in_hex(MAXGT);
  return "(CODESPACE (CTRANGE H 1 H $gc))\n";
}

sub j48_character {
  my ($t, $gc, $gx, $ss, $fid, $sx, $jx, @cnks);
  foreach $gc (1 .. MAXGT) {
    (($ss = sjposition($gc)) =~ m/-/) or $ss = GETA;
    (($fid, $sx) = $ss =~ m/^(..)-(....)$/) or die $ss;
    $fid -= 1; $gx = in_hex($gc);
    $jx = in_hex(in_jis(sjis(hex($sx))));
    $t = ($fid > 0) ? "(SELECTFONT D $fid)" : "";
    $t = "(CHARACTER H $gx (MAP $t (SETCHAR H $jx) ) )\n";
    push(@cnks, $t);
  }
  return join('', @cnks);
}

##------ zgu encoding

sub zgu_generate {
  my ($zvp, $vf, $tfm, $map);
  $map = zgu_genmap();
  $zvp = join('', u_zvp_prologue(""), u_mapfont(1),
                  u_type(), zgu_space($map), zgu_character($map));
  write_whole_file("gtmc-r-zgu.zvp", $zvp);
  exit;
}

sub zgu_genmap {
  my ($t, $uc, $fid, %u2j, %map);
  foreach $t (1410 .. 4374, 4418 .. 7807) {
    $u2j{in_ucs($t)} = in_jis($t);
  }
  foreach $fid (0 .. (MAXFN + MAXKFN - 1)) {
    $t = $fid * 0x10000;
    foreach $uc (keys %u2j) {
      $map{$t + $uc} = sprintf("%d-%04X", $fid, $u2j{$uc});
    }
  }
  return \%map;
}


sub zgu_space {
  my ($map) = @_; my ($t);
  $t = join("\n", map { sprintf("X%04X", $_) }
                     (sort { $a <=> $b } (keys %$map)));
  return "(CODESPACE $t)\n";
}

sub zgu_character {
  my ($map) = @_; my ($t, $fid, $jx, $uc, $ux, @ucs, @cnks);
  @ucs = sort { $a <=> $b } (keys %$map);
  foreach $uc (@ucs) {
    (defined($t = $map->{$uc})) or next;
    ($fid, $jx) = split(m/-/, $t); $ux = in_hex($uc);
    $t = ($fid > 0) ? "(SELECTFONT D $fid)" : "";
    $t = "(CHARACTER H $ux (MAP $t (SETCHAR H $jx) ) )\n";
    push(@cnks, $t);
  }
  return join('', @cnks);
}

##------ common subproc

sub u_zvp_prologue {
  my ($ncs) = @_;
  return <<"END";
(VTITLE GTMINCHO)
(FAMILY GTMINCHO)
(FACE F MRR)
(CODINGSCHEME $ncs)
(DESIGNSIZE R 10.0)
(CHECKSUM O 0)
(FONTDIMEN
   (SLANT R 0.0)
   (SPACE R 0.0)
   (STRETCH R 0.1)
   (SHRINK R 0.0)
   (XHEIGHT R 1.0)
   (QUAD R 1.0)
   (EXTRASPACE R 0.25)
   (EXTRASTRETCH R 0.2)
   (EXTRASHRINK R 0.125)
   )
END
}

sub u_mapfont {
  my ($sw) = @_; my ($n, $fid, $fnam, @c, @cnks);
  @c = ($sw) ? ('k1', 'k2') : (); $fid = 0;
  foreach $n (1 .. MAXFN, @c) {
    if ($n =~ m/^k/) {
      $fnam = "gtmc-r-g$n";
    } else {
      $fnam = ($n > 1) ? sprintf("gtmc-r-g%02d", $n) :
                "r-gtmc-r-jy1";
    }
    push(@cnks, <<"END");
(MAPFONT D $fid
   (FONTNAME $fnam)
   (FONTDSIZE R 10.0)
   (FONTAT R 1.0)
   )
END
    $fid++;
  }
  return join('', @cnks);
}

sub u_type {
  return <<"END";
(TYPE D 0
   (CHARWD R 1.0)
   (CHARHT R 0.88)
   (CHARDP R 0.12)
   )
END
}


##------ go to main
main();
## EOF
